@@ -27,6 +27,7 @@ enum {
 	CR_FD_MNTS,
 	CR_FD_USERNS,
 	CR_FD_TIMENS,
+	CR_FD_PIDNS,
 
 	_CR_FD_IPCNS_FROM,
 	CR_FD_IPC_VAR,
@@ -67,6 +68,8 @@ enum {
 	CR_FD_FILE_LOCKS,
 	CR_FD_SECCOMP,
 	CR_FD_MEMFD_INODE,
+	CR_FD_BPFMAP_FILE,
+	CR_FD_BPFMAP_DATA,
 	_CR_FD_GLOB_TO,
 
 	CR_FD_TMPFS_IMG,
