@@ -7,25 +7,24 @@
 #include "images/vma.pb-c.h"
 
 #include <sys/mman.h>
+#include <string.h>
 
 struct vm_area_list {
-	struct list_head	h;
-	unsigned		nr;
-	unsigned int		nr_aios;
-	unsigned long		priv_size; /* nr of pages in private VMAs */
-	unsigned long		priv_longest; /* nr of pages in longest private VMA */
-	unsigned long		shared_longest; /* nr of pages in longest shared VMA */
+	struct list_head	h;			/* list of VMAs */
+	unsigned		nr;			/* nr of all VMAs in the list */
+	unsigned int		nr_aios;		/* nr of AIOs VMAs in the list */
+	union {
+		unsigned long	nr_priv_pages;		/* dmp: nr of pages in private VMAs */
+		unsigned long	rst_priv_size;		/* rst: size of private VMAs */
+	};
+	unsigned long		nr_priv_pages_longest;	/* nr of pages in longest private VMA */
+	unsigned long		nr_shared_pages_longest;/* nr of pages in longest shared VMA */
 };
 
-#define VM_AREA_LIST(name)	struct vm_area_list name = { .h = LIST_HEAD_INIT(name.h), .nr = 0, }
-
 static inline void vm_area_list_init(struct vm_area_list *vml)
 {
+	memset(vml, 0, sizeof(*vml));
 	INIT_LIST_HEAD(&vml->h);
-	vml->nr = 0;
-	vml->priv_size = 0;
-	vml->priv_longest = 0;
-	vml->shared_longest = 0;
 }
 
 struct file_desc;
