@@ -357,7 +357,7 @@ def pb2dict(pb, pretty=False, is_hex=False):
         else:
             d_val = _pb2dict_cast(field, value, pretty, is_hex)
 
-        d[field.name] = d_val
+        d[field.name] = d_val.decode() if type(d_val) == bytes else d_val
     return d
 
 
