@@ -9,14 +9,13 @@ wget -nv https://download.opensuse.org/repositories/devel:kubic:libcontainers:st
 # this is a podman packaging bug (https://github.com/containers/libpod/issues/4747)
 apt-get -y purge docker-ce
 
-apt-get install -qq \
+./apt-install \
     apt-transport-https \
     ca-certificates \
     curl \
     software-properties-common
 
-apt-get update -qq
-apt-get install -qqy podman containernetworking-plugins
+./apt-install podman containernetworking-plugins
 
 export SKIP_TRAVIS_TEST=1
 
@@ -32,37 +31,38 @@ podman --storage-driver vfs info
 
 criu --version
 
+# shellcheck disable=SC2016
 podman run --name cr -d docker.io/library/alpine /bin/sh -c 'i=0; while true; do echo $i; i=$(expr $i + 1); sleep 1; done'
 
 sleep 1
-for i in `seq 20`; do
+for i in $(seq 20); do
 	echo "Test $i for podman container checkpoint"
 	podman exec cr ps axf
 	podman logs cr
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "1" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "1" ]
 	podman container checkpoint cr
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "0" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "0" ]
 	podman ps -a
 	podman container restore cr
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "1" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "1" ]
 	podman logs cr
 done
 
-for i in `seq 20`; do
+for i in $(seq 20); do
 	echo "Test $i for podman container checkpoint --export"
 	podman ps -a
 	podman exec cr ps axf
 	podman logs cr
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "1" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "1" ]
 	podman container checkpoint -l --export /tmp/chkpt.tar.gz
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "0" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "0" ]
 	podman ps -a
 	podman rm -fa
 	podman ps -a
 	podman container restore --import /tmp/chkpt.tar.gz
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "1" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "1" ]
 	podman container restore --name cr2 --import /tmp/chkpt.tar.gz
-	[ `podman ps -f name=cr2 -q -f status=running | wc -l` -eq "1" ]
+	[ "$(podman ps -f name=cr2 -q -f status=running | wc -l)" -eq "1" ]
 	podman ps -a
 	podman logs cr
 	podman logs cr2
@@ -70,7 +70,7 @@ for i in `seq 20`; do
 	podman rm -fa
 	podman ps -a
 	podman container restore --import /tmp/chkpt.tar.gz
-	[ `podman ps -f name=cr -q -f status=running | wc -l` -eq "1" ]
+	[ "$(podman ps -f name=cr -q -f status=running | wc -l)" -eq "1" ]
 	podman ps -a
 	rm -f /tmp/chkpt.tar.gz
 done
